@@ -3,6 +3,7 @@ package MyApp;
 use strict;
 use warnings;
 use Kossy;
+use Cookie::Baker;
 
 get "/" => sub {
     my ( $self, $c )  = @_;
@@ -18,6 +19,12 @@ router [qw/GET POST/] => "/bar" => sub {
     my ( $self, $c )  = @_;
     $c->response->body("ok");
 };
+
+get '/set_cookie' => sub {
+    my ( $self, $c )  = @_;
+    $c->response->cookies->{foo} = '123456';
+    $c->response->body("cookies are baked");
+};
  
 1;
 
